/*
 * rgbled.cpp
 *
 * Created: 02.08.2012 19:30:18
 *  Author: Unknown
 */ 

#include <math.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "hw/usiTwiSlave.h"
#include "hw/RgbLed.h"
#include "hw/clock.h"
#include "FadeTween.h"

#define TWI_BUF_LEN 16
#define BUF_R 0x08
#define BUF_G 0x09
#define BUF_B 0x0A
#define BUF_FADE_L 0x0B
#define BUF_FADE_H 0x0C
#define BUF_APPLY 0x0D // 0xFF to apply
uint8_t eeTWI_ADDR EEMEM = 0x50;

/* Vars and external methods */
RgbLed led;
FadeTween tween;
volatile uint8_t twibuf[TWI_BUF_LEN];

/* Called whenever a TWI byte is received */
void onTWI(uint8_t addr, uint8_t data) {
	if (addr < 0x08) {
		twibuf[0] = 'R'; twibuf[1] = 'G'; twibuf[2] = 'B'; twibuf[3] = ' ';
		twibuf[4] = 'v'; twibuf[5] = 'l'; twibuf[6] = 'm'; twibuf[7] = 'l';
	} else if (addr == 0x0D && data == 0xFF) {
		// apply changes	
		uint16_t fadeTime = twibuf[BUF_FADE_L] | (twibuf[BUF_FADE_H] << 8);
		tween.setNext(twibuf[BUF_R], twibuf[BUF_G], twibuf[BUF_B], fadeTime);
	}	
}

/* Setup methods */
inline void setupWDT() {
	MCUSR = 0;
	wdt_enable(WDTO_1S);	
	wdt_reset();
}
inline void setupPowersaving() {
	PORTB |= _BV(PB5); // pull-up for unused pins.
	ADCSRA &= ~_BV(ADEN); // disable adc
	ACSR |= _BV(ACD); // disable acd
	PRR |= _BV(PRADC); // power saving adc
	set_sleep_mode(SLEEP_MODE_IDLE);
}
inline void setupTWI() {
	// Clear the TWI buf
	for (uint8_t i = 0; i < TWI_BUF_LEN; i++)
		twibuf[i] = 0x00;
		
	// Read the address from eeprom
	uint8_t addr = eeprom_read_byte(&eeTWI_ADDR);	
	if (addr == 0x00 || addr == 0xFF) 
		addr = 0x50;
	addr <<= 1;
	
	// Init TWI
	onTWI(0, 0);
	usiTwiSlaveInit(addr, &twibuf[0], TWI_BUF_LEN);
	usiTwiCallback(onTWI);
}	
	
/* Main */
int main(void)
{
	// Setup everything
	setupWDT();
	setupPowersaving();
	setupTWI();
	setupClocks();
	
	led = RgbLed();
	led.start();
	
	tween = FadeTween();
	sei();

	// Do some startup seq (todo: soft fade)
	led.setColor(0xFF, 0x00, 0x00);
	_delay_ms(200);
	led.setColor(0x00, 0xFF, 0x00);
	_delay_ms(200);
	led.setColor(0x00, 0x00, 0xFF);
	_delay_ms(200);
	led.setColor(0x00, 0x00, 0x00);
	
	// Main loop
	// 100 kHz = ok
	// 125 kHz = nok
	while (1) {
		wdt_reset();
		tween.update();
		led.setColor(tween.R, tween.G, tween.B);
	}		
}